@@ -26,11 +26,15 @@ from bzrlib import (
     foreign,
     lockable_files,
     lockdir,
+    repository,
     revision,
     tests,
     trace,
+    vf_repository,
     )
 
+from bzrlib.repofmt import groupcompress_repo
+
 # This is the dummy foreign revision control system, used 
 # mainly here in the testsuite to test the foreign VCS infrastructure.
 # It is basically standard Bazaar with some minor modifications to 
@@ -92,32 +96,81 @@ class DummyForeignVcsBranch(branch.BzrBranch6,foreign.ForeignBranch):
         self._base = a_bzrdir.transport.base
         self._ignore_fallbacks = False
         self.bzrdir = a_bzrdir
-        foreign.ForeignBranch.__init__(self, 
+        foreign.ForeignBranch.__init__(self,
             DummyForeignVcsMapping(DummyForeignVcs()))
-        branch.BzrBranch6.__init__(self, _format, _control_files, a_bzrdir, 
+        branch.BzrBranch6.__init__(self, _format, _control_files, a_bzrdir,
             *args, **kwargs)
 
+    def _get_checkout_format(self):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repository uses weaves.
+        """
+        return self.bzrdir.checkout_metadir()
 
-class InterToDummyVcsBranch(branch.GenericInterBranch,
-                            foreign.InterToForeignBranch):
+    def import_last_revision_info_and_tags(self, source, revno, revid,
+                                           lossy=False):
+        interbranch = InterToDummyVcsBranch(source, self)
+        result = interbranch.push(stop_revision=revid, lossy=True)
+        if lossy:
+            revid = result.revidmap[revid]
+        return (revno, revid)
+
+
+class DummyForeignCommitBuilder(vf_repository.VersionedFileRootCommitBuilder):
+
+    def _generate_revision_if_needed(self):
+        mapping = DummyForeignVcsMapping(DummyForeignVcs())
+        if self._lossy:
+            self._new_revision_id = mapping.revision_id_foreign_to_bzr(
+                (str(self._timestamp), str(self._timezone), "UNKNOWN"))
+            self.random_revid = False
+        elif self._new_revision_id is not None:
+            self.random_revid = False
+        else:
+            self._new_revision_id = self._gen_revision_id()
+            self.random_revid = True
+
+
+class DummyForeignVcsRepository(groupcompress_repo.CHKInventoryRepository,
+    foreign.ForeignRepository):
+    """Dummy foreign vcs repository."""
+
+
+class DummyForeignVcsRepositoryFormat(groupcompress_repo.RepositoryFormat2a):
+
+    repository_class = DummyForeignVcsRepository
+    _commit_builder_class = DummyForeignCommitBuilder
+
+    def get_format_string(self):
+        return "Dummy Foreign Vcs Repository"
+
+    def get_format_description(self):
+        return "Dummy Foreign Vcs Repository"
+
+
+class InterToDummyVcsBranch(branch.GenericInterBranch):
 
     @staticmethod
     def is_compatible(source, target):
         return isinstance(target, DummyForeignVcsBranch)
 
-    def push(self, overwrite=False, stop_revision=None):
-        raise errors.NoRoundtrippingSupport(self.source, self.target)
-
-    def lossy_push(self, stop_revision=None):
+    def push(self, overwrite=False, stop_revision=None, lossy=False):
+        if not lossy:
+            raise errors.NoRoundtrippingSupport(self.source, self.target)
         result = branch.BranchPushResult()
         result.source_branch = self.source
         result.target_branch = self.target
         result.old_revno, result.old_revid = self.target.last_revision_info()
         self.source.lock_read()
         try:
+            graph = self.source.repository.get_graph()
             # This just handles simple cases, but that's good enough for tests
             my_history = self.target.revision_history()
-            their_history = self.source.revision_history()
+            if stop_revision is None:
+                stop_revision = self.source.last_revision()
+            their_history = list(graph.iter_lefthand_ancestry(stop_revision,
+                (revision.NULL_REVISION,)))
+            their_history.reverse()
             if their_history[:min(len(my_history), len(their_history))] != my_history:
                 raise errors.DivergedBranches(self.target, self.source)
             todo = their_history[len(my_history):]
@@ -129,7 +182,7 @@ class InterToDummyVcsBranch(branch.GenericInterBranch,
                     return (tree.get_file(file_id), None)
                 tree.get_file_with_stat = get_file_with_stat
                 new_revid = self.target.mapping.revision_id_foreign_to_bzr(
-                    (str(rev.timestamp), str(rev.timezone), 
+                    (str(rev.timestamp), str(rev.timezone),
                         str(self.target.revno())))
                 parent_revno, parent_revid= self.target.last_revision_info()
                 if parent_revid == revision.NULL_REVISION:
@@ -173,7 +226,8 @@ class DummyForeignVcsBranchFormat(branch.BzrBranchFormat6):
         super(DummyForeignVcsBranchFormat, self).__init__()
         self._matchingbzrdir = DummyForeignVcsDirFormat()
 
-    def open(self, a_bzrdir, name=None, _found=False, found_repository=None):
+    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
+            found_repository=None):
         if not _found:
             raise NotImplementedError
         try:
@@ -208,6 +262,10 @@ class DummyForeignVcsDirFormat(bzrdir.BzrDirMetaFormat1):
     def get_branch_format(self):
         return DummyForeignVcsBranchFormat()
 
+    @property
+    def repository_format(self):
+        return DummyForeignVcsRepositoryFormat()
+
     def initialize_on_transport(self, transport):
         """Initialize a new bzrdir in the base directory of a Transport."""
         # Since we don't have a .bzr directory, inherit the
@@ -240,6 +298,11 @@ class DummyForeignVcsDir(bzrdir.BzrDirMeta1):
         self._control_files = lockable_files.LockableFiles(self.transport,
             "lock", lockable_files.TransportLock)
 
+    def create_workingtree(self):
+        # dirstate requires a ".bzr" entry to exist
+        self.root_transport.put_bytes(".bzr", "foo")
+        return super(DummyForeignVcsDir, self).create_workingtree()
+
     def open_branch(self, name=None, unsupported=False, ignore_fallbacks=True):
         if name is not None:
             raise errors.NoColocatedBranchSupport(self)
@@ -249,6 +312,9 @@ class DummyForeignVcsDir(bzrdir.BzrDirMeta1):
         """Produce a metadir suitable for cloning with."""
         return bzrdir.format_registry.make_bzrdir("default")
 
+    def checkout_metadir(self):
+        return self.cloning_metadir()
+
     def sprout(self, url, revision_id=None, force_new_repo=False,
                recurse='down', possible_transports=None,
                accelerator_tree=None, hardlink=False, stacked=False,
@@ -262,12 +328,15 @@ class DummyForeignVcsDir(bzrdir.BzrDirMeta1):
 
 
 def register_dummy_foreign_for_test(testcase):
-    controldir.ControlDirFormat.register_format(DummyForeignVcsDirFormat)
-    testcase.addCleanup(controldir.ControlDirFormat.unregister_format,
-                        DummyForeignVcsDirFormat)
     controldir.ControlDirFormat.register_prober(DummyForeignProber)
     testcase.addCleanup(controldir.ControlDirFormat.unregister_prober,
         DummyForeignProber)
+    repository.format_registry.register(DummyForeignVcsRepositoryFormat())
+    testcase.addCleanup(repository.format_registry.remove,
+            DummyForeignVcsRepositoryFormat())
+    branch.format_registry.register(DummyForeignVcsBranchFormat())
+    testcase.addCleanup(branch.format_registry.remove,
+            DummyForeignVcsBranchFormat())
     # We need to register the optimiser to make the dummy appears really
     # different from a regular bzr repository.
     branch.InterBranch.register_optimiser(InterToDummyVcsBranch)
@@ -284,6 +353,10 @@ class DummyForeignProber(controldir.Prober):
             raise errors.NotBranchError(path=transport.base)
         return DummyForeignVcsDirFormat()
 
+    @classmethod
+    def known_formats(cls):
+        return set([DummyForeignVcsDirFormat()])
+
 
 class ForeignVcsRegistryTests(tests.TestCase):
     """Tests for the ForeignVcsRegistry class."""
@@ -302,8 +375,9 @@ class ForeignVcsRegistryTests(tests.TestCase):
         reg = foreign.ForeignVcsRegistry()
         vcs = DummyForeignVcs()
         reg.register("dummy", vcs, "Dummy VCS")
-        self.assertEquals((("some", "foreign", "revid"), DummyForeignVcsMapping(vcs)),
-                          reg.parse_revision_id("dummy-v1:some-foreign-revid"))
+        self.assertEquals((
+            ("some", "foreign", "revid"), DummyForeignVcsMapping(vcs)),
+            reg.parse_revision_id("dummy-v1:some-foreign-revid"))
 
 
 class ForeignRevisionTests(tests.TestCase):
@@ -377,7 +451,7 @@ class DummyForeignVcsTests(tests.TestCaseWithTransport):
         source_tree = self.make_branch_and_tree("source")
         target_tree = self.make_branch_and_tree("target", 
             format=DummyForeignVcsDirFormat())
-        pushresult = source_tree.branch.lossy_push(target_tree.branch)
+        pushresult = source_tree.branch.push(target_tree.branch, lossy=True)
         self.assertEquals(revision.NULL_REVISION, pushresult.old_revid)
         self.assertEquals(revision.NULL_REVISION, pushresult.new_revid)
         self.assertEquals({}, pushresult.revidmap)
@@ -391,7 +465,7 @@ class DummyForeignVcsTests(tests.TestCaseWithTransport):
             format=DummyForeignVcsDirFormat())
         target_tree.branch.lock_write()
         try:
-            pushresult = source_tree.branch.lossy_push(target_tree.branch)
+            pushresult = source_tree.branch.push(target_tree.branch, lossy=True)
         finally:
             target_tree.branch.unlock()
         self.assertEquals(revision.NULL_REVISION, pushresult.old_revid)
