@@ -141,24 +141,35 @@ added:
   hell\u00d8
 """.encode('utf8') in template)
 
-    def make_do_nothing_editor(self):
+    def make_do_nothing_editor(self, basename='fed'):
         if sys.platform == "win32":
-            f = file('fed.bat', 'w')
+            name = basename + '.bat'
+            f = file(name, 'w')
             f.write('@rem dummy fed')
             f.close()
-            return 'fed.bat'
+            return name
         else:
-            f = file('fed.sh', 'wb')
+            name = basename + '.sh'
+            f = file(name, 'wb')
             f.write('#!/bin/sh\n')
             f.close()
-            os.chmod('fed.sh', 0755)
-            return './fed.sh'
+            os.chmod(name, 0755)
+            return './' + name
 
     def test_run_editor(self):
         self.overrideEnv('BZR_EDITOR', self.make_do_nothing_editor())
         self.assertEqual(True, msgeditor._run_editor(''),
                          'Unable to run dummy fake editor')
 
+    def test_parse_editor_name(self):
+        """Correctly interpret names with spaces.
+
+        See <https://bugs.launchpad.net/bzr/+bug/220331>
+        """
+        self.overrideEnv('BZR_EDITOR',
+            '"%s"' % self.make_do_nothing_editor('name with spaces'))
+        self.assertEqual(True, msgeditor._run_editor('a_filename'))    
+
     def make_fake_editor(self, message='test message from fed\\n'):
         """Set up environment so that an editor will be a known script.
 
@@ -333,6 +344,18 @@ if len(sys.argv) == 2:
         self.assertRaises(errors.BadCommitMessageEncoding,
                           msgeditor.edit_commit_message, '')
 
+    def test_set_commit_message_no_hooks(self):
+        commit_obj = commit.Commit()
+        self.assertIs(None,
+            msgeditor.set_commit_message(commit_obj))
+
+    def test_set_commit_message_hook(self):
+        msgeditor.hooks.install_named_hook("set_commit_message",
+                lambda commit_obj, existing_message: "save me some typing\n", None)
+        commit_obj = commit.Commit()
+        self.assertEquals("save me some typing\n",
+            msgeditor.set_commit_message(commit_obj))
+
     def test_generate_commit_message_template_no_hooks(self):
         commit_obj = commit.Commit()
         self.assertIs(None,
@@ -357,11 +380,5 @@ class TestPlatformErrnoWorkarounds(TestCaseInTempDir):
         ERROR_BAD_EXE_FORMAT = 193
         file("textfile.txt", "w").close()
         e = self.assertRaises(WindowsError, subprocess.call, "textfile.txt")
-        # Python2.4 used the 'winerror' as the errno, which confuses a lot of
-        # our error trapping code. Make sure that we understand the mapping
-        # correctly.
-        if sys.version_info >= (2, 5):
-            self.assertEqual(e.errno, errno.ENOEXEC)
-            self.assertEqual(e.winerror, ERROR_BAD_EXE_FORMAT)
-        else:
-            self.assertEqual(e.errno, ERROR_BAD_EXE_FORMAT)
+        self.assertEqual(e.errno, errno.ENOEXEC)
+        self.assertEqual(e.winerror, ERROR_BAD_EXE_FORMAT)
